//-----------------------------------------------------------------------------
// Project            	RenderKit
// Class		G3DMatrix4f
// Creator            	Philippe C.D. Robert
// Maintainer         	Philippe C.D. Robert
// Creation Date      	Thu Sep  9 11:05:08 CEST 1999
//
// Copyright (c) Philippe C.D. Robert
//
// The GeometryKit is free software; you can redistribute it and/or modify it 
// under the terms of the GNU LGPL Version 2 as published by the Free 
// Software Foundation
//
// $Id: G3DMatrix4f.h,v 1.8 2001/01/05 20:50:48 robert Exp $
//-----------------------------------------------------------------------------

#ifndef __G3DMatrix4f_h_INCLUDE
#define __G3DMatrix4f_h_INCLUDE

#import <Foundation/Foundation.h>
#import <GeometryKit/G3DDefs.h>

@class G3DMatrix3f;
@class G3DTuple3f;
@class G3DTuple4f;
@class G3DVector3f;
@class G3DVector4f;
@class G3DQuaternion;

@interface G3DMatrix4f : NSObject <NSCoding, NSCopying>
{
  @public

  float matrix[16];

  @protected

  int _mode;           // G3D_MatrixType

  float _t[16];
  float _i[16];
}

//-----------------------------------------------------------------------------
// class methods
//-----------------------------------------------------------------------------

+ (void)initialize;

+ (G3DMatrix4f *)matrix4f;
+ (G3DMatrix4f *)matrix4fWithMatrix4f:(G3DMatrix4f *)aMatrix;
+ (G3DMatrix4f *)matrix4fWithElements:(float *)vals;
+ (G3DMatrix4f *)matrix4fWithRotation:(G3DMatrix4f *)vals translation:(G3DVector3f *)t scale:(float)factor;
+ (G3DMatrix4f *)matrix4fWithQuaternion:(G3DQuaternion *)q translation:(G3DVector3f *)t scale:(float)s;

//-----------------------------------------------------------------------------
// init and free
//-----------------------------------------------------------------------------

- (id)init;
- (id)initWithMatrix4f:(G3DMatrix4f *)aMatrix;
- (id)initWithElements:(float *)vals;

- (id)initWithRotation:(G3DMatrix4f *)vals translation:(G3DVector3f *)t scale:(float)factor;
// self = S * R * T

- (id)initWithQuaternion:(G3DQuaternion *)q translation:(G3DVector3f *)t scale:(float)s;

- (void)dealloc;

//-----------------------------------------------------------------------------
// math
//-----------------------------------------------------------------------------

- (void)addMatrix:(G3DMatrix4f *)a;
- (void)subMatrix:(G3DMatrix4f *)a;

- (void)multiplyByMatrix:(G3DMatrix4f *)a;
- (void)multiplyByMatrix:(G3DMatrix4f *)a and:(G3DMatrix4f *)b;
- (void)multiplyByMatrix:(G3DMatrix4f *)a andTransposed:(G3DMatrix4f *)b;
- (void)multiplyByTransposedMatrix:(G3DMatrix4f *)a and:(G3DMatrix4f *)b;
- (void)multiplyByTransposedMatrix:(G3DMatrix4f *)a andTransposed:(G3DMatrix4f *)b;

- (void)multiplyByMatrices:(NSArray *)matrices;

- (BOOL)invert;
- (G3DMatrix4f *)invertedMatrix;

- (void)transpose;
- (G3DMatrix4f *)transposedMatrix;

- (void)addScalar:(float)scalar;
- (void)subScalar:(float)scalar;

- (void)multiplyByScalar:(float)scalar;
- (void)divideByScalar:(float)scalar;

- (void)multiplyRow:(int)pos byValue:(float)val;
- (void)multiplyCol:(int)pos byValue:(float)val;

- (void)addElements:(float *)vals toRow:(int)rowPos;
- (void)addElements:(float *)vals toColumn:(int)colPos;

- (void)subElements:(float *)vals fromRow:(int)rowPos;
- (void)subElements:(float *)vals fromColumn:(int)colPos;

- (float)determinant;

- (G3DTuple4f *)vectorByPreMultiplying:(G3DTuple4f *)aTuple;
  // Returns aTuple * self;

- (G3DTuple4f *)vectorByPostMultiplying:(G3DTuple4f *)aTuple;
  // Returns self * aTuple

//-----------------------------------------------------------------------------
// accessor methods
//-----------------------------------------------------------------------------

- (G3DVector3f *)translation;
- (void)setTranslation:(G3DVector3f *)t;

- (G3DQuaternion *)quaternion;
- (void)setQuaternion:(G3DQuaternion *)aQuat;

- (void)setMatrix:(G3DMatrix4f *)aMatrix;

- (G3DMatrix3f *)rotation;
- (void)setRotation:(G3DMatrix3f *)rot;

- (void)setXRotation:(float)angle;
- (void)setYRotation:(float)angle;
- (void)setZRotation:(float)angle;
- (void)setRotation:(float)angle axis:(float *)axis;
- (void)setRotation:(float)angle axisVector:(G3DVector3f *)vec;

- (G3DVector3f *)scale;
- (void)setScale:(G3DVector3f *)scale;

- (void)setRotation:(G3DMatrix4f *)vals translation:(G3DVector3f *)t scale:(float)factor;

- (void)setQuaternion:(G3DQuaternion *)q translation:(G3DVector3f *)t scale:(float)s;

- (void)setElements:(float *)vals;
- (float *)elements;

- (G3DVector4f *)rowVectorAtIndex:(int)row;
- (G3DVector4f *)columnVectorAtIndex:(int)col;

- (void)setElement:(float)val atRow:(int)row column:(int)col;
- (float)elementAtRow:(int)row column:(int)col;

//-----------------------------------------------------------------------------
// misc
//-----------------------------------------------------------------------------

- (BOOL)isEqualToMatrix:(G3DMatrix4f *)aMatrix;

- (NSString *)description;

//-----------------------------------------------------------------------------
// NSCoding
//-----------------------------------------------------------------------------

- (void)encodeWithCoder:(NSCoder *)aCoder;
- (id)initWithCoder:(NSCoder *)aCoder;

//-----------------------------------------------------------------------------
// NSCopying
//-----------------------------------------------------------------------------

- (id)copyWithZone:(NSZone *)zone;

@end

extern NSString *G3DMatrix4fException;

#endif




